/*******************************************************************************
 * Copyright (c) 2013 Richard M. Katzwer/Benjamin Brooks/Dilip Abreu/Yuliy Sannikov.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v2.1
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * 
 * Contributors:
 *     Richard M. Katzwer/Benjamin Brooks/Dilip Abreu/Yuliy Sannikov - initial API and implementation
 ******************************************************************************/
package edu.princeton.repeatedgames.rgsolve.example;
import edu.princeton.repeatedgames.rgsolve.RGSolution;
import edu.princeton.repeatedgames.rgsolve.RGSolve;
import edu.princeton.repeatedgames.rgsolve.components.SolutionGraph;
import edu.princeton.repeatedgames.rgsolve.games.BimatrixGame;
import edu.princeton.repeatedgames.rgsolve.games.Game;

/**
 * A class with some simple examples
 * to get a Java programmer started
 * using the rgsolve package!
 */
public class RGSolveExample {

	private RGSolveExample() { }
	
    @SuppressWarnings("javadoc")
	public static void main(String[] args) {
        
        /* --------------------------------- *
         * Example 1: The Prisoner's Dilemma *
         * --------------------------------- */
        
        // Define payoffs
        double[][] G1 = { { 2, -1},
                          { 3,  1} };

        double[][] G2 = { { 2,  3},
                          {-1,  1} };
        
        double discount = .5;
        
        // Instantiate the game
        BimatrixGame pdgame = new BimatrixGame(
            G1, G2, discount, null, "The The Prisoner's Dilemma"
        );
        
        // Instantiate the solver for this game,
        // which will use the default parameters,
        // and print output to the console
        RGSolve rgsolve = new RGSolve(pdgame);
        
        // Solve the game and store the solution
        RGSolution pdsoln = rgsolve.solveGame();

        // create a solution panel to view the solution
        // to the game
        SolutionGraph.showSolutionPanel(pdsoln);
        
        /* --------------------------------- *
         * Example 2: A Bertrand Game        *
         * --------------------------------- */
        
        // instantiate an anonymous class for our Bertrand game
        int m1 = 100, m2 = 100;
        double discount2 = .5;
        final double A = 10, B = 2;    // Q = A - B * P
        final double MC = .5;          // marginal cost
        
        /** 
         * A simple perfect-substitutes duopoly Bertrand
         * game with demand function Q = A - B * P
         * and marginal cost MC */
        Game bertrand = new Game(m1, m2, discount2, "100x100 Bertrand") {

            @Override
            public double payoff1(int a1, int a2) {
                double p1 = A / B * a1/(m1()-1);
                double p2 = A / B * a2/(m2()-1);
                if(p1 > p2)
                    return 0;
                else {
                    double Q = Math.max( A - B * p1, 0 );
                    if(p1 == p2)
                        return Q / 2 * (p1 - MC);
                    else
                        return Q * (p1 - MC);
                }
            }

            @Override
            public double payoff2(int a1, int a2) {
                double p1 = A / B * a1/(m1()-1);
                double p2 = A / B * a2/(m2()-1);
                if(p2 > p1)
                    return 0;
                else {
                    double Q = Math.max( A - B * p2, 0 );
                    if(p2 == p1)
                        return Q / 2 * (p2 - MC);
                    else
                        return Q * (p2 - MC);
                }
            }
            
        };
        
        // Instantiate the solver for this game,
        // which will use the default parameters,
        // and print output to the console
        rgsolve = new RGSolve(bertrand);
        
        // Solve the game and store the solution
        RGSolution bertrandSoln = rgsolve.solveGame();

        // create a solution panel to view the solution
        // to the game
        SolutionGraph.showSolutionPanel(bertrandSoln);
 
    }

}
